---
categories: sysadmin web
title: Howto Cron
...

* man 5 crontab : <https://manpages.debian.org/jessie/cron/crontab.5.en.html>

Le service **cron** (diminutif de *crontab*, pour **c**h**ron**o **tab**le) est le planificateur de tâches standard sur les systèmes UNIX/Linux. Il permet le déclenchement de commandes à des dates récurrentes (à la minute près).

# Installation

En général le paquet **cron** est installé et le service tourne déjà :

~~~
# systemctl status cron
● cron.service - Regular background program processing daemon
   Loaded: loaded (/lib/systemd/system/cron.service; enabled; vendor preset: enabled)
     Docs: man:cron(8)
 Main PID: 492 (cron)
   CGroup: /system.slice/cron.service
           └─492 /usr/sbin/cron -f
~~~

# Configuration

La configuration se trouve dans le fichier `/etc/crontab` :

~~~
SHELL=/bin/sh
PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

# Example of job definition:
# .---------------- minute (0 - 59)
# |  .------------- hour (0 - 23)
# |  |  .---------- day of month (1 - 31)
# |  |  |  .------- month (1 - 12) OR jan,feb,mar,apr ...
# |  |  |  |  .---- day of week (0 - 6) (Sunday=0 or 7) OR sun,mon,tue,wed,thu,fri,sat
# |  |  |  |  |
# *  *  *  *  * user-name command to be executed
17 *    * * *   root    cd / && run-parts --report /etc/cron.hourly
25 6    * * *   root    test -x /usr/sbin/anacron || ( cd / && run-parts --report /etc/cron.daily )
47 6    * * 7   root    test -x /usr/sbin/anacron || ( cd / && run-parts --report /etc/cron.weekly )
52 6    1 * *   root    test -x /usr/sbin/anacron || ( cd / && run-parts --report /etc/cron.monthly )
~~~

On trouve notamment des dates fixées pour les actions définies dans les répertoires `/etc/cron.{hourly,daily,weekly,monthly}/` : nous conseillons de modifier ces horaires fixes, pour éviter des déclenchements simultanés sur plusieurs serveurs.

On peut également modifier certaines options via le fichier `/etc/default/cron`, notamment la journalisation  : par défaut `-L 5` enregistre le démarrage de chaque cron et les éventuelles erreurs, on peut passer à `-L 15` pour avoir un maximum d'informations notamment la date de fin de chaque cron.


# Utilisation

## /etc/cron.{hourly,daily,weekly,monthly}/ et /etc/cron.d/

Pour lancer un script en tant que _root_ à une date fixée par le système, il suffit de le rendre exécutable et de le placer dans l'un des répertoires `/etc/cron.{hourly,daily,weekly,monthly}/` en fonction de la fréquence souhaitée.

Pour une fréquence personnalisée, on peut utiliser le répertoire `/etc/cron.d/` où il faudra indiquer en plus la fréquence et l'utilisateur qui doit lancer la commande, par exemple :

~~~
30 05 * * 0 www-data /usr/local/adm/savelog-weekly
~~~

> **Attention** : En cas d'erreur de syntaxe dans un script, tous les autres scripts du même répertoire ne seront pas lancés ! Il faut donc réserver cette utilisation à des scripts bien vérifiés (c'est notamment utilisé par les packages Debian).

> **Attention** : Les noms de scripts contenant des points sont ignorés par cron. On pourra par exemple utiliser cette particularité pour désactiver un script, en ajoutant .old ou .disabled à la fin de son nom.


## crontab -e

Pour ajouter un cron manuellement, on conseille d'utiliser la commande :

~~~
$ crontab -e
# crontab -u jdoe -e
~~~

L'un des avantages de cette commande est que la syntaxe sera vérifiée en quittant l'éditeur.

Pour lancer une commande à une fréquence personnalisée, la syntaxe est :

~~~
minutes heures jours mois jour/semaine commande
~~~

On peut utiliser `*` pour toutes les occurrences (tous les jours, ou toutes les heures, etc.). On peut également utiliser des listes à virgule (`3,5,7`) ainsi des intervalles (`2-6`) ; on peut aussi avoir des intervalles réguliers (`*/15`).

Exemples de crontab :

~~~
0,30,45,51 * * * * /usr/local/adm/send-data
*/15 * * * * /usr/local/adm/check-nis 1>/dev/null 2>&1
00 01 * * * nice -10 find /inf -name core -exec rm -f {} \;
10 03 * * 1-6 nice -10 /usr/local/adm/sauvegarde-daily
30 05 * * 0 /usr/local/adm/savelog-weekly
30 06 1 * * /usr/local/adm/savelog-monthly
00 00 1 1 * /usr/local/bin/happy-new-year
@daily /usr/local/bin/minuit-check
~~~

On peut utiliser les mots clés `@hourly`/`@daily`/`@weekly`/`@monthly` pour lancer à la première minute de chaque heure/jour/semaine/mois (à ne pas confondre avec les répertoires /etc/cron.{hourly,daily,weekly,monthly}/ qui sont lancés à des dates particulières).

Pour [plus d'informations sur la syntaxe](https://manpages.debian.org/jessie/cron/crontab.5.en.html) :

~~~
$ man 5 crontab
~~~

Pour lister les crons, on peut utiliser la commande :

~~~
$ crontab -l
# crontab -u jdoe -l
~~~

Pour supprimer tous les crons en place, on peut utiliser `crontab -r`… mais c'est assez rare qu'on l'utilise. Il faut surtout se méfier de ne pas le taper par erreur à la place de `crontab -e` !

Pour plus de détails sur l'utilisation de la commande `crontab` en tant qu'utilisateur, voir [HowtoCrontab](), et pour des recommandations sur l'écriture de scripts à mettre en crontab voir [TipsDevWeb#scripts-en-crontab]().

### variables

Plusieurs variables peuvent être utilisées dans une crontab :

`PATH` : pour redéfinir le PATH des commandes lancés

`SHELL` : par défaut c'est /bin/sh qui est utilisé, on peut vouloir utiliser /bin/bash par exemple

`MAILTO` : lorsqu'une commande génère une sortie (stdout/stderr), elle est envoyée par email à l'utilisateur. On peut changer l'email destinataire avec cette variable. On pourra notamment mettre une liste à virgule pour envoyer à plusieurs adresses, ou alors `MAILTO=""` pour empêcher l'envoi d'un message.

Ces variables peuvent être définies plusieurs fois, notamment `MAILTO` qui pourra précéder chaque ligne de cron :

~~~
MAILTO=jdoe@example.com,alert@example.com
@daily df -h
MAILTO=""
@hourly systemctl restart javajob
MAILTO=alert@example.com
@weekly /usr/local/bin/security-check
~~~

### /var/spool/cron/crontabs/

Les crons positionnés avec `crontab -e` sont stockés dans le répertoire `/var/spool/cron/crontabs/`
C'est notamment utile pour réaliser des sauvegardes des crons lancés sur un serveur.


# log2mail

On peut ajouter la configuration suivante au logiciel log2mail pour détecter d'éventuelles erreurs de syntaxe :

~~~
file = /var/log/cron.log
  pattern = "Error"
  mailto = root
~~~


# FAQ

## Peut-on écrire un cron dans /etc/crontab ?

Si vous ajoutez une ligne de cron dans `/etc/crontab` elle sera exécutée en _root_ : mais nous déconseillons fortement cette utilisation car ce fichier de configuration doit rester intact autant que possible, en général il faut éviter de lancer un cron en tant que _root_ et enfin la moindre erreur de syntaxe va provoquer le non lancement de tous les autres scripts !

## Changement d'heure

En France, lors d'un changement d'heure d'été ou d'hiver, on peut imaginer qu'un script lancé entre 2h et 3h du matin se lancera deux fois ou zéro fois selon les cas. Mais sous Debian, lorsque *cron* détecte un saut important dans l'heure locale, il va prévenir ce comportement en lançant les crons qu'une seule fois (passage à l'heure d'hiver) ou juste après le changement d'heure (passage à l'heure d'été). Par précaution, nous évitons tout de même de mettre des crons entre 2h et 3h du matin.

## Échappement de caractères

Prenons l'exemple d'une copie horodatée de /etc chaque heure :

~~~
@hourly root tar -cf /home/backup/etc-$(date +%H).tar /etc
~~~

Celle-ci **ne s'exécutera pas**. En effet, le caractère `%` est interprété comme un saut de ligne et votre commande sera incomplète, et dans ce cas non fonctionnelle.

Il faut alors simplement **échapper** le `%` avec un `\` :

~~~
@hourly root tar -cf /home/backup/etc-$(date +\%H).tar /etc
~~~